
package org.apidesign.apifest08.currency;

import java.math.BigDecimal;

/**
 * Result of currency conversion. Holds converted value and remainder.
 * <p>
 * <em>Converter</em> describes value converted to <em>target</em> value. <em>Remainder</em> describes
 * how much from original <em>amount</em> was not possible to convert. Converted never loses any (small) money
 * in conversion error (rounding), but instead of rounding is converts only as much as possible and keeps rest as remainder.
 * 
 * @author arnostvalicek
 */
public class ConversionResult {
    private BigDecimal converted;
    private BigDecimal remainder;
    
    /**
     * Get converted value.
     * @return Returns converted value.
     */
    public BigDecimal getConverted() {
        return converted;
    }

    void setConverted(BigDecimal converted) {
        this.converted = converted;
    }

    
    /**
     * Get remainder of conversion. Remainder is set if part of converted amount which can't be converted 
     * because this target currency precision can't handle small numbers. Remainder value is in <em>from currency</em>
     * <p>
     * Converter never loses any precision in conversion. Remainer describes how much of amount can't be converted. 
     * If we substract <em>remainder</em> from <em>amount</em> we will be able to get exact conversion.
     * 
     * @return Returns remainder of conversion.
     */
    public BigDecimal getRemainder() {
        return remainder;
    }

    void setRemainder(BigDecimal remainder) {
        this.remainder = remainder;
    }

}
