/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.apidesign.apifest08.test;

import java.math.BigDecimal;
import java.util.Currency;
import junit.framework.Assert;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.DelegatingConvertor;
import org.apidesign.apifest08.currency.MonetaryAmount;

/**
 * A delegating convertor that checks preconditions and postconditions.
 * Useful for testing.
 * @author jdvorak
 */
public class ContractImposingDelegatingConvertor extends DelegatingConvertor {

    public ContractImposingDelegatingConvertor( final Convertor underlyingConvertor ) {
        super( underlyingConvertor );
    }

    @Override
    public ConversionResult convert( final ConversionRequest req ) {
        Assert.assertNotNull( "The request", req );
        final ConversionResult result = super.convert( req );
        Assert.assertNotNull( "Result of the convert() call", result );
        final MonetaryAmount netAmount = result.getNetAmount();
        if ( netAmount != null ) {
            Assert.assertEquals( "Converted to a different currency than specified in the request", req.getTgtCurrency(), netAmount.getCurrency() );
        }
        return result;
    }

    /**
     * Do some tests on our own.
     * @return this
     */
    public Convertor test() {
        try {
            final Currency aCurrency = Currency.getInstance( "EUR" );
            new ConversionRequest( new MonetaryAmount( BigDecimal.ONE, aCurrency ), aCurrency );
            Assert.fail( "Should have thrown an IllegalArgumentException" );
        } catch ( final IllegalArgumentException e ) {
            Assert.assertEquals( "Cannot request conversion from EUR to EUR", e.getMessage() );
        }
        return this;
    }

}
